/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * robotviz 0.1                                                          *
 * Copyright (C) 2013  Holger Jost                                       *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef _ROBOTVIZ_H_
#define _ROBOTVIZ_H_

#include <iostream>
#include <string>
#include <list>
#include <tuple>
#include <vector>
#include <algorithm>
#include <unistd.h>

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_rotozoom.h>

using namespace std;

enum Tile   {tile_empty, tile_left, tile_right};
enum Robot  {robot_red, robot_green, robot_blue, robot_yellow};
enum Symbol {sun_red, sun_green, sun_blue, sun_yellow,
             moon_red, moon_green, moon_blue, moon_yellow,
             star_red, star_green, star_blue, star_yellow,
             planet_red, planet_green, planet_blue, planet_yellow,
             random_robot};
enum Arrow  {arrow_red_north, arrow_green_north, arrow_blue_north, arrow_yellow_north,
             arrow_red_east, arrow_green_east, arrow_blue_east, arrow_yellow_east,
             arrow_red_south, arrow_green_south, arrow_blue_south, arrow_yellow_south,
             arrow_red_west, arrow_green_west, arrow_blue_west, arrow_yellow_west};

class Item{
public:
	Item();
	Item(int x, int y, Symbol item);
	int x, y;
	Symbol symbol;
};

class RobotViz{
	public:
		static void parseDim(string input);
		static void parseBarrier(string input);
		static void parsePosition(string input);
		static void parseTarget(string input);
		static void parseMove(string input);
		static void constructBoard();
		static bool sortMoveTuple(const tuple<int,int,string,int> &lhs, const tuple<int,int,string,int> &rhs);
		static void calcPositions();
		static void initSDL();
		static string getPath();
		static void setupImages();
		static void pollEvents();
		static void draw();
	private:
		static vector<tuple<int,int,string>>        barrierList;
		static vector<tuple<int,int,int,int>>       positionList;
		static vector<tuple<string,int,int>>        targetList;
		static vector<tuple<int,int,string,int>>    moveList;

		static int width;
		static int height;

		static int steps;

		static bool boardOnly;

		static int wOffset;
		static int hOffset;
		static double origRatio;
		static double currRatio;

		static int tileSize;

		static int screenWidth;
		static int screenHeight;
		static SDL_Surface *screen;
		static SDL_Surface *board;
		static SDL_Surface *tile[3];
		static SDL_Surface *robot[4];
		static SDL_Surface *wallNorth, *wallWest;
		static SDL_Surface *item[17];
		static SDL_Surface *arrow[16];

		static list<pair<int, int>> wallsNorth;
		static list<pair<int, int>> wallsWest;

		static Item targetItem;

		static int numRobots;
		static vector<pair<int, int>> robotPositions[4];
		static unsigned step;
};

#endif
